<?php

namespace Packages\Product\Http\Controllers;

use App\Http\Controllers\Controller;
use Facade\Ignition\Support\Packagist\Package;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Support\Facades\Facade;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;
use Packages\Product\Http\Requests\ProductRequest;
use Packages\Product\Http\Requests\ProductRequestEdit;
use Packages\Product\Models\Product;
use Illuminate\Http\Request;
use App\Models\Category;
use App\Models\Upload;
use phpDocumentor\Reflection\Types\Compound;



class ProductController extends Controller
{
    public function __construct()
    {

    }

//  *******************************************show  page product ************
    public function index()
    {
        $categories = Category::orderBy('id', 'DESC')->get();

        $products = Product::query()->orderBy('id', 'DESC');

        if (request()->has('productCategory') && request('productCategory') != '')
            $products = $products->whereHas('categories', function (Builder $query) {
                $query->whereRaw('id = ?', [request('productCategory')]);
            });

        if (request()->has('productStatus') && request('productStatus') != '')
            $products = $products->whereRaw('status = ?', [request('productStatus')]);

        if (request()->has('productName') && request('productName') != '')
            $products = $products->whereRaw('title = ?', [request('productName')]);
        if (request()->has('creatorId') && request('creatorId') != '')
            $products = $products->whereHas('user', function (Builder $query) {
                $query->whereRaw('name= ?', [request('creatorId')]);
            });
        $products = $products->paginate();


        return view('product::index', compact('products', 'categories'));

    }

//**************************************************** page create products *****************
    public function create()
    {
        $categories = Category::orderBy('parent_id')->where('parent_id', 0)->get();

        return view('product::product.create', compact('categories'));
    }

// *********************************************************save products ******************

    public function store(ProductRequest $request)
    {
        /*foreach ($request->file('gallery_image') as $item) {
            echo '<pre>';
            print_r($item);
            echo '</pre>';
        }
        die();*/

        $request->merge([
            'price' => preg_replace('/[^0-9]+/', '', $request->price),
            'sale_price' => preg_replace('/[^0-9]+/', '', $request->sale_price),
        ]);
        $data = [
            'title' => $request->title,
             'slug' => $request->slug,
            'sku' => $request->sku,
            'ReleaseStatus' => $request->ReleaseStatus,
            'price' => $request->price,
            'sale_price' => $request->sale_price,
            'type' => $request->type,
            'status' => $request->status,
            'discription' => $request->discription,
            'creator_id' => auth()->user()->id
        ];
//        \DB::enableQueryLog();
        $product = Product::create($data);
        $product->categories()->sync($request->categories);
//        dd(\DB::getQueryLog());

        if ($request->has('gallery_image')) {
            $file = $request->only('gallery_image', 'captionGallery', 'descriptionGallery');
            $type = 'gallery_image';
            $diskName = 'product';
            $this->uploadGallery($file, $diskName, $product, $type);

        }
        if ($request->has('featured_image')) {
            $info = $request->only(['featureDescription', 'featureCaption']);
            $file = $request->file('featured_image');
            $type = 'featured_image';
            $diskName = 'product';
            $this->uploadFeature($file, $diskName, $product, $type, $info);

        }

        $msg = 'ذخیره محصول با موفقیت انجام شد ';
        return redirect(route('products.edit', $product->id))->with('success', $msg);
    }

    // ******************************************************edit page product*********************************************

    public function edit(Product $product)
    {
        $categories = Category::where('parent_id', 0)->orderBy('title')->get();
        return view('product::product.edit', compact('product', 'categories'));
    }

// *******************************************update page product*****************************************

    public function update(ProductRequestEdit $request, Product $product)
    {

        $request->merge([
            'price' => preg_replace('/[^0-9]+/', '', $request->price),
            'sale_price' => preg_replace('/[^0-9]+/', '', $request->sale_price),
        ]);
//        'categories' => $request->categories,
        $data = [
            'title' => $request->title,
            'slug' => $request->slug,
            'sku' => $request->sku,
            'price' => $request->price,
            'sale_price' => $request->sale_price,
            'type' => $request->type,
            'ReleaseStatus' => $request->ReleaseStatus,
            'status' => $request->status,
            'discription' => $request->discription,
            'creator_id' => auth()->user()->id,

        ];
        $product->update($data);

        $product->categories()->sync($request->categories);
        if ($request->has('gallery_image')) {
            $file = $request->only('gallery_image', 'captionGallery', 'descriptionGallery');
            $type = 'gallery_image';
            $diskName = 'product';
            $this->updateGallery($file, $diskName, $product, $type);

        }
        if ($request->has('featured_image')) {
            $info = $request->only(['featureDescription', 'featureCaption']);
            $file = $request->file('featured_image');
            $type = 'featured_image';
            $diskName = 'product';
            $this->uploadFeature($file, $diskName, $product, $type, $info);

        }


        $msg = 'ویرایش محصول با موفقیت انجام شد ';

        return redirect(route('products.index'))->with('success', $msg);
    }

    //***************************** delete products*****************************************

    public function destroy(Product $product)
    {

        $product->delete();
        {
            return back();
        }
    }
//****************upload images in the product************************
    public function uploadFeature($file, $diskName, $product, $type,$info)
    {

        $fileExtension = $file->getClientOriginalExtension();

        $fileMimeType = $file->getMimeType();

        $afterDiskRoot = '/' . jdate()->format('Y') . '/' . jdate()->format('m');

        $fileName = jdate(time())->format('Ymd') . '_' . $file->getClientOriginalName();

        $filePath = storage_path('app/public/' . $diskName . $afterDiskRoot . '/' . $fileName);

        if (File::exists($filePath)) {
            $fileName = time() . '_' . $fileName;
        }

        $upload = $file->storeAs($afterDiskRoot, $fileName, $diskName);


        $uploadData = [
            'name' => $fileName,
            'path' => $upload,
            'mime_type' => $fileMimeType,
            'extension' => $fileExtension,
           'type' => $type,
            'descriptionImg' => $info['featureDescription'],
            'caption' => $info['featureCaption'],


        ];


        $uploaded = $product->uploads()->create($uploadData);


    }

    public function updateFeature($file, $diskName, $product, $type,$info)
    {

        $fileExtension = $file->getClientOriginalExtension();

        $fileMimeType = $file->getMimeType();

        $afterDiskRoot = '/' . jdate()->format('Y') . '/' . jdate()->format('m');

        $fileName = jdate(time())->format('Ymd') . '_' . $file->getClientOriginalName();

        $filePath = storage_path('app/public/' . $diskName . $afterDiskRoot . '/' . $fileName);

        if (File::exists($filePath)) {
            $fileName = time() . '_' . $fileName;
        }

        $upload = $file->storeAs($afterDiskRoot, $fileName, $diskName);


        $uploadData = [
            'name' => $fileName,
            'path' => $upload,
            'mime_type' => $fileMimeType,
            'extension' => $fileExtension,
            'type' => $type,
            'descriptionImg' => $info['featureDescription'],
            'caption' => $info['featureCaption'],


        ];


        $uploaded = $product->uploads()->update($uploadData);


    }

//************************delete image in th product**********************
    public function destroyFile(Upload $upload)
    {
        $upload->delete();
        $msg = 'حذف عکس محصول با موفقیت انجام شد ';
        session()->flash('image_deleted',$msg);
        return redirect()->back();
    }
    public function uploadGallery($files, $diskName, $product, $type)
    {
//        dd($files);
        foreach ($files['gallery_image'] as $key => $file) {
            $fileExtension = $file->getClientOriginalExtension();

            $fileMimeType = $file->getMimeType();

            $afterDiskRoot = '/' . jdate()->format('Y') . '/' . jdate()->format('m');

            $fileName = jdate(time())->format('Ymd') . '_' . $file->getClientOriginalName();

            $filePath = storage_path('app/public/' . $diskName . $afterDiskRoot . '/' . $fileName);

            if (File::exists($filePath)) {
                $fileName = time() . '_' . $fileName;
            }

            $upload = $file->storeAs($afterDiskRoot, $fileName, $diskName);


            $uploadData = [
                'name' => $fileName,
                'path' => $upload,
                'mime_type' => $fileMimeType,
                'extension' => $fileExtension,
                'type' => $type,
                'descriptionImg' => $files['descriptionGallery'][$key],
                'caption' => $files['captionGallery'][$key],
            ];

            $uploaded = $product->uploads()->create($uploadData);

        }

    }



    public function updateGallery($files, $diskName, $product, $type)
    {
//        dd($files);
        foreach ($files['gallery_image'] as $key => $file) {
            $fileExtension = $file->getClientOriginalExtension();

            $fileMimeType = $file->getMimeType();

            $afterDiskRoot = '/' . jdate()->format('Y') . '/' . jdate()->format('m');

            $fileName = jdate(time())->format('Ymd') . '_' . $file->getClientOriginalName();

            $filePath = storage_path('app/public/' . $diskName . $afterDiskRoot . '/' . $fileName);

            if (File::exists($filePath)) {
                $fileName = time() . '_' . $fileName;
            }

            $upload = $file->storeAs($afterDiskRoot, $fileName, $diskName);


            $uploadData = [
                'name' => $fileName,
                'path' => $upload,
                'mime_type' => $fileMimeType,
                'extension' => $fileExtension,
                'type' => $type,
                'descriptionImg' => $files['descriptionGallery'][$key],
                'caption' => $files['captionGallery'][$key],
            ];

            $uploaded = $product->uploads()->update($uploadData);

        }

    }
    public function myformProduct(Request $request)
    {
        $messages = [
            'title.required'   => 'وارد کردن نام الزامی است!',
            'price.required'    => 'وارد کردن قیمت الزامی است!',
            'slug.required'        => 'وارد کردن slug الزامی است!',
            'type.required'           => 'وارد کردن نوع الزامی است!',
            'status.required'      => 'وارد کردن وضعیت الزامی است!',
            'discription.required'      => 'وارد کردن توضیحات الزامی است!',
            'categories.required'      => 'وارد کردن دسته بندی الزامی است!',
            'featured_image.required'      => 'وارد کردن تصوی شاخص الزامی است!',

        ];
        $validator = Validator::make($request->all(), [
            'title'    => 'required',
            'price'     => 'required',
            'slug'         => 'required',
            'type'       => 'required',
            'status'       => 'required',
            'discription'       => 'required',
            'categories'       => 'required',
            'featured_image'       => 'required',

        ], $messages);

        if ($validator->passes()) {
            return response()->json(['success'=>'Added new records.']);
        }
        return response()->json(['error'=>$validator->errors()->all()]);
    }
    public function SaveFormProductByAjax()
    {
        $title = $this->input->post('title');
        $price = $this->input->post('price');
        $slug = $this->input->post('slug');
        $type= $this->input->post('type');
        $status = $this->input->post('status');
        $discription = $this->input->post('discription');
        $categories = $this->input->post('categories');
        $featured_image = $this->input->post('featured_image');

        $data=array('title'=>$title,"price"=>$price,"slug"=>$slug,"type"=>$type,"status"=>$status,"discription"=>$discription,"categories"=>$categories,"featured_image"=>$featured_image);
        DB::table('enquiry')->insert($data);
        echo "Record inserted successfully";
    }
}
